library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY SYNCHROX IS
GENERIC (NCMPT: integer := 10); -- nombre de bits des compteurs lignes et colonnes -1
				      -- identiques pour simplifier
	PORT
(ck, r_in, v_in, b_in				: IN	STD_LOGIC;
r_out, v_out, b_out, hor_sync, vert_sync		: OUT	STD_LOGIC;
pixel_ln, pixel_col			: OUT STD_LOGIC_VECTOR(NCMPT DOWNTO 0));
	END SYNCHROX;

-- la position des pixels affichs est donne  chaque instant par pixel_ln et pixel_col
-- les sorties couleurs r_out, v_out et b_out donnent la valeur (1 ou 0) du pixel pour la position.
-- les entres de couleur r_in, v_in et b_in devront tre synchronises avec la position des pixels.

ARCHITECTURE arch OF SYNCHROX IS

-- le comptage commenant  0,
-- les constantes sont diminues d'une unit par rapport  la grandeur reprsente

	CONSTANT NPA		: INTEGER :=799;--nombre de pixels affichs par lignes
	CONSTANT DSH		: INTEGER :=855;--dbut du top synchro horizontale
	CONSTANT FSH		: INTEGER :=975;--fin du top synchro horizontale
	CONSTANT NPT		: INTEGER :=1039;--nombre de pixels total par lignes
	
	CONSTANT NLA		: INTEGER :=599;--nombre de lignes affiches
	CONSTANT DSV		: INTEGER :=636;--dbut du top synchro verticale
	CONSTANT FSV		: INTEGER :=642;--fin du top synchro horizontale
	CONSTANT NLT		: INTEGER :=665;--nombre de lignes total			


SIGNAL video_on, video_on_v, video_on_h 	: STD_LOGIC;
SIGNAL h_cmpt	  		: STD_LOGIC_VECTOR(NCMPT DOWNTO 0);
SIGNAL v_cmpt 			: STD_LOGIC_VECTOR(NCMPT DOWNTO 0);

BEGIN

-- autorisation d'affichage uniquement dans la zone image verticale et horizontale
video_on <= video_on_H AND video_on_V;


PROCESS
BEGIN
	WAIT UNTIL(ck 'EVENT) AND (ck ='1');

-- gnration de la synchronisation et de l'affichage horizontal

-- h_cmpt compte (NPA+1) pixels d'affichage plus la synchronisation horizontale
-- le comptage commence au dbut de l'affichage d'une ligne

-- pixel_col donne la position des pixels affichs
-- pixel_col reste  NPA lors de signaux de synchronisation

-- video_on dfinit la zone d'affichage sur la ligne

--  hor_sync   ------------------------------------------------__________---------
--  h_cmpt     0                    NPA                      DSH          FSH    NPT
--  pixel_col   0                    NPA                      NPA          NPA    NPA 
--  video_on_h ---------------------______________________________


	IF (h_cmpt >= NPT) 		THEN 	h_cmpt <= (others => '0');
						pixel_col <= (others => '0');
						hor_sync<='1';
						video_on_h <= '1';

	ELSIF (h_cmpt < NPA)		THEN 	h_cmpt <= h_cmpt + 1;
						pixel_col <= h_cmpt +1;
						hor_sync<='1';
						video_on_h <= '1';

	ELSIF (h_cmpt >= NPA AND h_cmpt < DSH)
					THEN 	h_cmpt <= h_cmpt + 1;
						hor_sync<='1';
						video_on_h <= '0';

	ELSIF (h_cmpt >= DSH AND h_cmpt < FSH)
					THEN 	h_cmpt <= h_cmpt + 1;
						hor_sync<='0';
   						video_on_h <= '0';

	ELSE					h_cmpt <= h_cmpt + 1;
						hor_sync<='1';
						video_on_h <= '0';
	
END IF;


-- v_cmpt compte (NLA+1) lignes d'affichage plus la synchronisation verticale
-- l'incrmentation du compteur ne doit se faire qu'en fin de ligne (h_cmpt=NPT)
-- pixel_ln donne la position des pixels affichs
-- video_on autorise ou non l'affichage
  
--  vert_sync      -------------------------------------_______------------
--  v_cmpt         0                         NLA      DSV     FSV       NLT
--  pixel_ln        0                         NLA      NLA      NLA        NLA
--  video_on_v     -------------------------______________________


	IF (v_cmpt >= NLT) AND (h_cmpt >= NPT) 	THEN	v_cmpt <= (others => '0');
								pixel_ln <= (others => '0');
								vert_sync <= '1';
								video_on_v <= '1';

	ELSIF (v_cmpt < NLA) AND (h_cmpt >= NPT) THEN 	v_cmpt<= v_cmpt + 1;
								pixel_ln <= v_cmpt +1;
								vert_sync<='1';
   								video_on_v <= '1';
										
	ELSIF (v_cmpt >= NLA AND v_cmpt < DSV) AND (h_cmpt >= NPT) 
							THEN 	v_cmpt<= v_cmpt + 1;
								vert_sync<='1';
   								video_on_v <= '0';
 	   										
	ELSIF (v_cmpt >= DSV AND v_cmpt < FSV) AND (h_cmpt >= NPT)
							THEN 	v_cmpt<= v_cmpt + 1;
								vert_sync<='0';
   								video_on_v <= '0';

 	ELSIF (h_cmpt >= NPT)			THEN	v_cmpt<= v_cmpt + 1;
								vert_sync<='1';
   								video_on_v <= '0';

		END IF;

-- l'affichage n'est pas autoris pendant les temps de synchronisation
		r_out <= r_in AND video_on;
		v_out <= v_in AND video_on;
		b_out <= b_in AND video_on;

	END PROCESS;

END arch;
